@@ -13,6 +13,7 @@
 # limitations under the License.
 #
 
+from ferry.fabric.com import robust_com
 import json
 import logging
 import os
@@ -103,88 +104,132 @@ class DockerCLI(object):
         self.port_flag = ' -p'
         self.expose_flag = ' -expose'
         self.volume_flag = ' -v'
+        self.cid_file = ' --cidfile'
         self.lxc_flag = ' -lxc-conf'
         self.disable_net = ' -n=false'
         self.host_flag = ' -h'
         self.fs_flag = ' -s'
         self.env_flag = ' -e'
         self.registry = registry
+        self.docker_user = 'root'
 
-    """
-    Get the backend driver docker is using. 
-    """
-    def get_fs_type(self):
+    def _execute_cmd(self, cmd, server=None, user=None, read_output=True):
+        """
+        Execute the command on the server via ssh. 
+        """
+
+        if not server:
+            # The server is not supplied, so just execute
+            # the command locally. 
+            proc = Popen(cmd, stdout=PIPE, stderr=PIPE, shell=True)
+        else:
+            # Do not store results in hosts file or warn about 
+            # changing ssh keys. Also use the key given to us by the fabric. 
+            flags = " -o ConnectTimeout=10 -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null "
+            flags += " -i " + self.key
+
+            # If the user is given explicitly use that. Otherwise use the
+            # default user (which is probably root). 
+            if user:
+                ip = user + '@' + server
+            else:
+                ip = self.docker_user + '@' + server
+            flags += " -t -t " + ip
+
+            # Wrap the command around an ssh command. 
+            ssh = 'ssh ' + flags + ' \'%s\'' % cmd
+            logging.warning(ssh)
+
+            # All the possible errors that might happen when
+            # we try to connect via ssh. 
+            if read_output:
+                out, err = robust_com(ssh)
+            else:
+                # The user does not want us to read the output.
+                # That means we can't really check for errors :(
+                proc = Popen(ssh, stdout=PIPE, stderr=PIPE, shell=True)
+
+        if read_output:
+            # Read both the standard out and error. 
+            return proc.stdout.read(), proc.stderr.read()
+        else:
+            # The user does not want to read the output.
+            return proc
+        
+    def get_fs_type(self, server=None):
+        """
+        Get the backend driver docker is using. 
+        """
         cmd = self.docker + ' ' + self.info_cmd + ' | grep Driver | awk \'{print $2}\''
         logging.warning(cmd)
 
-        output = Popen(cmd, stdout=PIPE, shell=True).stdout.read()
+        output, _ = self._execute_cmd(cmd)
         return output.strip()
         
-    """
-    Fetch the current docker version.
-    """
-    def version(self):
+    def version(self, server=None):
+        """
+        Fetch the current docker version.
+        """
         cmd = self.docker + ' ' + self.version_cmd + ' | grep Client | awk \'{print $3}\''
         logging.warning(cmd)
 
-        output = Popen(cmd, stdout=PIPE, shell=True).stdout.read()
+        output, _ = self._execute_cmd(cmd, server)
         return output.strip()
         
-    """
-    List all the containers. 
-    """
-    def list(self):
+    def list(self, server=None):
+        """
+        List all the containers. 
+        """
         cmd = self.docker + ' ' + self.ps_cmd + ' -q' 
         logging.warning(cmd)
 
-        output = Popen(cmd, stdout=PIPE, shell=True).stdout.read()
+        output, _ = self._execute_cmd(cmd, server)
         output = output.strip()
 
         # There is a container ID for each line
         return output.split()
 
-    """
-    List all images that match the image name
-    """
-    def images(self, image_name=None):
-        if not image_name:
-            cmd = self.docker + ' ' + self.images_cmd + ' | awk \'{print $1}\''
-            output = Popen(cmd, stdout=PIPE, shell=True).stdout.read()
-        else:
-            cmd = self.docker + ' ' + self.images_cmd + ' | awk \'{print $1}\'' + ' | grep ' + image_name
-            output = Popen(cmd, stdout=PIPE, shell=True).stdout.read()
+    def images(self, image_name=None, server=None):
+        """
+        List all images that match the image name
+        """
+        cmd = self.docker + ' ' + self.images_cmd + ' | awk \'{print $1}\''
+        if image_name:
+            cmd = cmd  + ' | grep ' + image_name
 
+        output, _ = self._execute_cmd(cmd, server)
         logging.warning(cmd)
         return output.strip()
-    """
-    Build a new image from a Dockerfile
-    """
-    def build(self, image, docker_file=None):
+
+    def build(self, image, docker_file=None, server=None):
+        """
+        Build a new image from a Dockerfile
+        """
         path = '.'
         if docker_file != None:
             path = docker_file
 
         cmd = self.docker + ' ' + self.build_cmd + ' -t %s %s' % (image, path)
         logging.warning(cmd)
-        output = Popen(cmd, stdout=PIPE, shell=True).stdout.read()
+        output, _ = self._execute_cmd(cmd, server)
 
     def _get_default_run(self, container):
         cmd = self.docker + ' ' + self.inspect_cmd + ' ' + container.container
         logging.warning(cmd)
 
-        output = Popen(cmd, stdout=PIPE, shell=True).stdout.read()
+        output, _ = self._execute_cmd(cmd)
         data = json.loads(output.strip())
         
         cmd = data[0]['Config']['Cmd']
         return json.dumps( {'Cmd' : cmd} )
 
-    def login(self, user, password, email, registry):
+    def login(self, user, password, email, registry, server=None):
         """
         Login to a remote registry. 
         """
         cmd = self.docker + ' ' + self.login_cmd + ' -u %s -p %s -e %s %s' % (user, password, email, registry)
         logging.warning(cmd)
-        output = Popen(cmd, stdout=PIPE, shell=True).stdout.read()
+        output, _ = self._execute_cmd(cmd)
         if output.strip() == "Login Succeeded":
             return True
         else:
@@ -213,7 +258,7 @@ class DockerCLI(object):
             pass
         return True
 
-    def push(self, image, registry=None):
+    def push(self, image, registry=None, server=None):
         """
         Push an image to a remote registry.
         """
@@ -222,24 +267,24 @@ class DockerCLI(object):
             new_image = "%s/%s" % (registry, raw_image_name)
             tag = self.docker + ' ' + self.tag_cmd + ' ' + image + ' ' + new_image
             logging.warning(tag)
-            Popen(tag, shell=True)
+            self._execute_cmd(tag, server, read_output=False)
         else:
             new_image = image
         push = self.docker + ' ' + self.push_cmd + ' ' + new_image
         logging.warning(push)
-        child = Popen(push, stdout=PIPE, stderr=PIPE, shell=True)
+        child = self._execute_cmd(push, server, read_output=False)
         return self._continuous_print(child, "uploading image...")
 
-    def pull(self, image):
+    def pull(self, image, server=None):
         """
         Pull a remote image to the local registry. 
         """
         pull = self.docker + ' ' + self.pull_cmd + ' ' + image
         logging.warning(pull)
-        child = Popen(pull, stdout=PIPE, stderr=PIPE, shell=True)
+        child = self._execute_cmd(pull, server, read_output=False)
         return self._continuous_print(child, "downloading image...")
 
-    def commit(self, container, snapshot_name):
+    def commit(self, container, snapshot_name, server=None):
         """
         Commit a container
         """
@@ -249,48 +294,52 @@ class DockerCLI(object):
         # Construct a new container using the given snapshot name. 
         cmd = self.docker + ' ' + self.commit_cmd + ' ' + run_cmd + ' ' + container.container + ' ' + snapshot_name
         logging.warning(cmd)
-        Popen(cmd, shell=True)
+        self._execute_cmd(cmd, server)
 
-    """
-    Stop a running container
-    """
-    def stop(self, container):
+    def stop(self, container, server=None):
+        """
+        Stop a running container
+        """
         cmd = self.docker + ' ' + self.stop_cmd + ' ' + container
         logging.warning(cmd)
-        output = Popen(cmd, stdout=PIPE, shell=True).stdout.read()
+        self._execute_cmd(cmd, server)
 
-    """
-    Remove a container
-    """
-    def remove(self, container):
+    def remove(self, container, server=None):
+        """
+        Remove a container
+        """
         cmd = self.docker + ' ' + self.rm_cmd + ' ' + container
         logging.warning(cmd)
-        output = Popen(cmd, stdout=PIPE, shell=True).stdout.read()
+        self._execute_cmd(cmd, server)
 
-    """
-    Start a stopped container. 
-    """
-    def start(self, container, service_type, keydir, keyname, privatekey, volumes, args):
+    def start(self, image, container, service_type, keydir, keyname, privatekey, volumes, args, server=None, user=None, inspector=None, background=False):
+        """
+        Start a stopped container. 
+        """
         cmd = self.docker + ' ' + self.start_cmd + ' ' + container
         logging.warning(cmd)
-        output = Popen(cmd, stdout=PIPE, shell=True).stdout.read()
+        
+        if background:
+            proc = self._execute_cmd(cmd, server, user, False)
+            container = None
+        else:
+            output, _ = self._execute_cmd(cmd, server, user, True)
+            container = output.strip()
 
         # Now parse the output to get the IP and port
-        container = output.strip()
-        return self.inspect(container = container, 
-                            keydir = keydir,
-                            keyname = keyname,
-                            privatekey = privatekey,
-                            volumes = volumes,
-                            service_type = service_type, 
-                            args = args)
+        return inspector.inspect(image = image,
+                                 container = container, 
+                                 keydir = keydir,
+                                 keyname = keyname,
+                                 privatekey = privatekey,
+                                 volumes = volumes,
+                                 service_type = service_type, 
+                                 args = args)
 
-    """
-    Run a command in a virtualized container
-    The Docker allocator will ignore subnet, volumes, instance_name, and key
-    information since everything runs locally. 
-    """
-    def run(self, service_type, image, volumes, keydir, keyname, privatekey, open_ports, host_map=None, expose_group=None, hostname=None, default_cmd=None, args=None, lxc_opts=None):
+    def run(self, service_type, image, volumes, keydir, keyname, privatekey, open_ports, host_map=None, expose_group=None, hostname=None, default_cmd=None, args=None, lxc_opts=None, server=None, user=None, inspector=None, background=False):
+        """
+        Start a brand new container
+        """
         flags = self.daemon 
 
         # Specify the hostname (this is optional)
@@ -336,21 +385,31 @@ class DockerCLI(object):
         if not default_cmd:
             default_cmd = ''
 
+        # The user does not want to print out the output (makes sense
+        # if the container "eats" up the physical network device). However
+        # we should still store the container ID somewhere. 
+        if background:
+            flags += self.cid_file + " /ferry/containers/container.pid"
+
         # Now construct the final docker command. 
         cmd = self.docker + ' ' + self.run_cmd + ' ' + flags + ' ' + image + ' ' + default_cmd
         logging.warning(cmd)
-        child = Popen(cmd, stdout=PIPE, stderr=PIPE, shell=True)
 
-        err = child.stderr.read().strip()
-        if re.compile('[/:\s\w]*Can\'t connect[\'\s\w]*').match(err):
-            logging.error("Ferry docker daemon does not appear to be running")
-            return None
-        elif re.compile('Unable to find image[\'\s\w]*').match(err):
-            logging.error("%s not present" % image)
-            return None
+        if background:
+            proc = self._execute_cmd(cmd, server, user, False)
+            container = None
+        else:
+            output, error = self._execute_cmd(cmd, server, user, True)
+            err = error.strip()
+            if re.compile('[/:\s\w]*Can\'t connect[\'\s\w]*').match(err):
+                logging.error("Ferry docker daemon does not appear to be running")
+                return None
+            elif re.compile('Unable to find image[\'\s\w]*').match(err):
+                logging.error("%s not present" % image)
+                return None
+            container = output.strip()
 
-        container = child.stdout.read().strip()
-        return self.inspect(container, keydir, keyname, privatekey, volumes, hostname, open_ports, host_map, service_type, args)
+        return inspector.inspect(image, container, keydir, keyname, privatekey, volumes, hostname, open_ports, host_map, service_type, args, server)
 
     def _get_lxc_net(self, lxc_tuples):
         for l in lxc_tuples:
@@ -359,15 +418,18 @@ class DockerCLI(object):
                 return ip
         return None
 
-    """
-    Inspect a container and return information on how
-    to connect to the container. 
-    """
-    def inspect(self, container, keydir=None, keyname=None, privatekey=None, volumes=None, hostname=None, open_ports=[], host_map=None, service_type=None, args=None):
-        cmd = self.docker + ' ' + self.inspect_cmd + ' ' + container
-        logging.warning(cmd)
+class DockerInspector(object):
+    def __init__(self, cli):
+        self.cli = cli
 
-        output = Popen(cmd, stdout=PIPE, shell=True).stdout.read()
+    def inspect(self, image, container, keydir=None, keyname=None, privatekey=None, volumes=None, hostname=None, open_ports=[], host_map=None, service_type=None, args=None, server=None):
+        """
+        Inspect a container and return information on how
+        to connect to the container. 
+        """
+        cmd = self.cli.docker + ' ' + self.cli.inspect_cmd + ' ' + container
+        logging.warning(cmd)
+        output, _ = self.cli._execute_cmd(cmd, server)
 
         data = json.loads(output.strip())
         instance = DockerInstance()
@@ -375,6 +437,13 @@ class DockerCLI(object):
         if type(data) is list:
             data = data[0]
 
+        # Check if the container is running. It is an error
+        # if the container is not running.
+        if not bool(data['State']['Running']):
+            logging.error("container for %s is not running" % image)
+            return None
+
+        # Otherwise start collecting the various container information. 
         instance.image = data['Config']['Image']
         instance.container = data['ID']
         instance.internal_ip = data['NetworkSettings']['IPAddress']
@@ -382,7 +451,7 @@ class DockerCLI(object):
         # If we've used the lxc config, then the networking information
         # will be located somewhere else. 
         if instance.internal_ip == "":
-            instance.internal_ip = self._get_lxc_net(data['HostConfig']['LxcConf'])
+            instance.internal_ip = self.cli._get_lxc_net(data['HostConfig']['LxcConf'])
 
         if hostname:
             instance.host_name = hostname
@@ -410,7 +479,7 @@ class DockerCLI(object):
         else:
             instance.volumes = data['Volumes']
 
-            
+        # Store the key information. 
         instance.keydir = keydir
         instance.keyname = keyname
         instance.privatekey = privatekey
